@@ -1,4 +1,4 @@
-from unittest import mock
+from unittest import mock  # pylint: disable=too-many-lines
 
 import pandas as pd
 import pyspark
@@ -950,44 +950,26 @@ class TestNode:  # pylint: disable=too-many-public-methods
             def t4(test):
                 return test
 
+    def test_run_parallel(self):
+        @node(type="pandas")
+        def t1():
+            return pd.DataFrame({"c1": [3]})
 
-# class TestNodeIntegration:
-#     """
-#     Higher level integration tests
-#     """
+        @node(type="pandas", dependencies=[t1])
+        def t2(t1):
+            return t1 + 1
 
-# def test_spark_sql(self, spark):
-#     """
-#     Simple pipeline to check that we can flick between regular dataframe and Spark SQL transformations.
-#     """
-#     # # FIXME- incomplete, need to get pyspark working locally again first
-#     # @node(
-#     #     type='pandas'
-#     # )
-#     # def t1():
-#     #     return pd.DataFrame({'name': ['Bob', 'Chris'], 'age': [10, 20]})
-#     #
-#     # @node(
-#     #     type='spark_sql',
-#     #     dependencies=[t1]
-#     # )
-#     # def t2(t1):
-#     #     return f'SELECT name, age + 1 FROM {t1}'
-#     #
-#     # @node(
-#     #     type='spark_sql',
-#     #     dependencies=[t2]
-#     # )
-#     # def t3(t2):
-#     #     return f'SELECT name, age + 1 FROM {t2}'
-#     #
-#     # @node(
-#     #     type='pandas',
-#     #     dependencies=[t3]
-#     # )
-#     # def t4(t3):
-#     #     t3['age'] += 1
-#     #     return t3
-#     #
-#     # result = t4.run(spark=spark)
-#     # pass
+        @node(type="pandas", dependencies=[t1])
+        def t3(t1):
+            return t1 * 10
+
+        @node(type="pandas", dependencies=[t1])
+        def t4(t1):
+            return t1**2
+
+        @node(type="pandas", dependencies=[t2, t3, t4])
+        def t5(t2, t3, t4):
+            return pd.concat([t2, t3, t4]).reset_index(drop=True)
+
+        result = t5.run(parallel=True)
+        assert_frame_equal(result, pd.DataFrame({"c1": [4, 30, 9]}))
